// Displays calls and their associated recordings in the grid. Allows the user
// to sort the entries, filter their search, and search for a specific person.
// Authors:
// Cameron Hayes
// Mac ...

unit View.Orders;

interface

uses
  System.SysUtils, System.Classes, Web, WEBLib.Graphics, WEBLib.Forms, WEBLib.Dialogs,
  Vcl.Controls, Vcl.StdCtrls, WEBLib.StdCtrls, WEBLib.Controls, WEBLib.Grids, WebLib.Lists,
  XData.Web.Client, WEBLib.ExtCtrls, DB, XData.Web.JsonDataset,
  XData.Web.Dataset, XData.Web.Connection, Vcl.Forms, DateUtils, WebAudio;

type
  TFViewOrders = class(TWebForm)
    XDataWebClient1: TXDataWebClient;
    XDataWebDataSet1: TXDataWebDataSet;
    lblEntries: TWebLabel;
    wcbPageSize: TWebComboBox;
    wcbSortBy: TWebComboBox;
    btnApply: TWebButton;
    edtSearch: TWebEdit;
    WebDataSource1: TWebDataSource;
    XDataWebDataSet1companyName: TStringField;
    XDataWebDataSet1jobName: TStringField;
    XDataWebDataSet1orderDate: TStringField;
    XDataWebDataSet1proofDue: TStringField;
    XDataWebDataSet1proofDone: TStringField;
    XDataWebDataSet1artDue: TStringField;
    XDataWebDataSet1artDone: TStringField;
    XDataWebDataSet1mountDue: TStringField;
    XDataWebDataSet1mountDone: TStringField;
    XDataWebDataSet1shipDue: TStringField;
    XDataWebDataSet1shipDone: TStringField;
    XDataWebDataSet1price: TStringField;
    XDataWebDataSet1ID: TStringField;
    XDataWebDataSet1qbRefNum: TStringField;
    XDataWebDataSet1colors: TStringField;
    procedure WebFormCreate(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnSearchClick(Sender: TObject);
  private
    FChildForm: TWebForm;
    procedure AddRowToTable(temp: string);
    procedure ClearTable();
    procedure GeneratePagination(TotalPages: Integer);
    function GenerateSearchOptions(): string;
    [async] procedure Search(searchOptions: string);
    [async] procedure GetCalls(searchOptions: string);
    [async] procedure getUser();
    var
      PageNumber: integer;
      PageSize: integer;
      TotalPages: integer;
      StartDate: string;
      EndDate: string;
      OrderBy: string;
      Caller: string;
  public

  end;

var
  FViewOrders: TFViewOrders;

implementation

uses
  JS, XData.Model.Classes,
  ConnectionModule, Auth.Service;

{$R *.dfm}

procedure TFViewOrders.WebFormCreate(Sender: TObject);
// Initializes important values:
// PageNumber: What page number the user is on IE 1: 1-10, 2: 11-20 etc
// TotalPages: Total number of pages returned from the search.
// PageSize: Number of entries per page.
begin
  DMConnection.ApiConnection.Connected := True;
  PageNumber := 1;
  TotalPages := 1; // Initial total pages
  wcbPageSize.Text := '10';
  getCalls(GenerateSearchOptions());
end;

procedure TFViewOrders.getUser();
var
  xdcResponse: TXDataClientResponse;
  userList, user: TJSObject;
  data: TJSArray;
begin
  xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.GetUsers',
        [JS.toString(AuthService.TokenPayload.Properties['user_name'])]));
  userList :=  TJSObject(xdcResponse.Result);
  data := TJSArray(userList['data']);
  user := TJSObject(data[0]);
end;


procedure TFViewOrders.AddRowToTable(temp: string);
// Adds rows to the table
// PhoneNumber: phone number of the location
// Caller: phone number of the caller
// Duration: duration of the call
// Transcript: transcription of the recording
// MediaUrl: Link to the recording
var
  NewRow, Cell, P, Button, Audio: TJSHTMLElement;
  colorObject: TJSObject;
  colorList: TJSArray;
  colors: integer;
begin
  NewRow := TJSHTMLElement(document.createElement('tr'));

  // Order ID Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Order ID');
  if XDataWebDataSet1ID.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1ID.Value;
  NewRow.appendChild(Cell);

  // Company Name Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Company Name');
  if XDataWebDataSet1companyName.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1companyName.Value;
  NewRow.appendChild(Cell);

  // Job Name Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Job Name');
  if XDataWebDataSet1jobName.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1jobName.Value;
  NewRow.appendChild(Cell);

  // Order Date Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Order Date');
  if XDataWebDataSet1orderDate.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1orderDate.Value;
  NewRow.appendChild(Cell);

  // Proof Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Proof Due');
  if XDataWebDataSet1proofDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1proofDue.Value;
  NewRow.appendChild(Cell);

  // Proof Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Proof Done');
  if XDataWebDataSet1proofDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1proofDone.Value;
  NewRow.appendChild(Cell);

  // Art Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Art Due');
  if XDataWebDataSet1artDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1artDue.Value;
  NewRow.appendChild(Cell);

  // Art Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Art Done');
  if XDataWebDataSet1artDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1artDone.Value;
  NewRow.appendChild(Cell);

  // Mount Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Mount Due');
  if XDataWebDataSet1mountDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1mountDue.Value;
  NewRow.appendChild(Cell);

  // Mount Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Mount Done');
  if XDataWebDataSet1mountDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1mountDone.Value;
  NewRow.appendChild(Cell);

  // Ship Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Ship Due');
  if XDataWebDataSet1shipDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1shipDue.Value;
  NewRow.appendChild(Cell);

  // Ship Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Ship Done');
  if XDataWebDataSet1shipDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1shipDone.Value;
  NewRow.appendChild(Cell);

  // Price Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Price');
  if XDataWebDataSet1price.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1price.Value;
  NewRow.appendChild(Cell);

  // QuickBooks Reference Number Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Quickbooks Reference Number');
  if XDataWebDataSet1qbRefNum.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1qbRefNum.Value;
  NewRow.appendChild(Cell);

  // Colors Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Colors');
  if XDataWebDataSet1colors.Value = '' then
    Cell.innerText := '0'
  else
  begin
    colorObject := TJSObject(TJSJSON.parse(XDataWebDataSet1colors.Value));
    colorList := TJSArray(colorObject['items']);
    colors := colorList.flength;
    Cell.innerText := IntToStr(colors);
  end;
  NewRow.appendChild(Cell);


  // Appends new rows to the table body
  TJSHTMLElement(document.getElementById('tblPhoneGrid').getElementsByTagName('tbody')[0]).appendChild(NewRow);
end;

procedure TFViewOrders.GeneratePagination(TotalPages: Integer);
// Generates pagination for the table.
// TotalPages: Total amount of pages generated by the search
var
  PaginationElement, PageItem, PageLink: TJSHTMLElement;
  I, Start, Finish: Integer;
begin
  PaginationElement := TJSHTMLElement(document.getElementById('pagination'));
  PaginationElement.innerHTML := ''; // Clear existing pagination

  // Previous Button
  PageItem := TJSHTMLElement(document.createElement('li'));
  PageItem.className := 'page-item';
  if PageNumber = 1 then
    PageItem.classList.add('disabled');
  PageLink := TJSHTMLElement(document.createElement('a'));
  PageLink.className := 'page-link';
  PageLink.innerText := 'Previous';
  PageLink.setAttribute('href', 'javascript:void(0)');
  PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
  begin
    if PageNumber > 1 then
    begin
      Dec(PageNumber);
      GetCalls(GenerateSearchOptions());
    end;
  end);
  PageItem.appendChild(PageLink);
  PaginationElement.appendChild(PageItem);

  // Page Numbers

  if PageNumber <= 4 then
  // If page number is low enough no early elipsis needed
  Begin
    Start := 2;
    Finish := 5;
  End
  else if (PageNumber >= (TotalPages - 3)) then
  // If page number is high enough no late elipsis needed
  begin
    Start := TotalPages - 3;
    Finish := TotalPages - 1;
  end
  else
  begin
    Start := PageNumber - 1;
    Finish := PageNumber + 1;
  end;

  PageItem := TJSHTMLElement(document.createElement('li'));
  PageItem.className := 'page-item';
  if 1 = PageNumber then
    PageItem.classList.add('selected-number'); // Add the selected-number class
  PageLink := TJSHTMLElement(document.createElement('a'));
  PageLink.className := 'page-link';
  PageLink.innerText := '1';
  PageLink.setAttribute('href', 'javascript:void(0)');
  PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
  var
    PageNum: Integer;
  begin
    PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
    PageNumber := PageNum;
    GetCalls(GenerateSearchOptions());
  end);
  PageItem.appendChild(PageLink);
  PaginationElement.appendChild(PageItem);

  // Adds Elipse to pagination if page number is too big
  if PageNumber > 4 then
  begin
    PageItem := TJSHTMLElement(document.createElement('li'));
    PageItem.className := 'page-item';
    PageItem.classList.add('disabled');
    PageLink := TJSHTMLElement(document.createElement('a'));
    PageLink.className := 'page-link';
    PageLink.innerText := '...';
    PageLink.setAttribute('href', 'javascript:void(0)');
    PageItem.appendChild(PageLink);
    PaginationElement.appendChild(PageItem);
  end;


  // Adds Page, page - 1, and page + 1 to pagination
  for I := Start  to Finish do
  begin
    if ( I > 1) and (I < TotalPages) then
    begin
      PageItem := TJSHTMLElement(document.createElement('li'));
      PageItem.className := 'page-item';
      if I = PageNumber then
        PageItem.classList.add('selected-number'); // Add the selected-number class
      PageLink := TJSHTMLElement(document.createElement('a'));
      PageLink.className := 'page-link';
      PageLink.innerText := IntToStr(I);
      PageLink.setAttribute('href', 'javascript:void(0)');
      PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
      var
        PageNum: Integer;
      begin
        PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
        PageNumber := PageNum;
        GetCalls(GenerateSearchOptions());
      end);
      PageItem.appendChild(PageLink);
      PaginationElement.appendChild(PageItem);
    end;
  end;

  // adds ellipse if number is too small
  if PageNumber < TotalPages - 4 then
  begin
    PageItem := TJSHTMLElement(document.createElement('li'));
    PageItem.className := 'page-item';
    PageItem.classList.add('disabled');
    PageLink := TJSHTMLElement(document.createElement('a'));
    PageLink.className := 'page-link';
    PageLink.innerText := '...';
    PageLink.setAttribute('href', 'javascript:void(0)');
    PageItem.appendChild(PageLink);
    PaginationElement.appendChild(PageItem);
  end;

  if TotalPages <> 1 then
  begin
    PageItem := TJSHTMLElement(document.createElement('li'));
    PageItem.className := 'page-item';
    if TotalPages = PageNumber then
          PageItem.classList.add('selected-number');
    PageLink := TJSHTMLElement(document.createElement('a'));
    PageLink.className := 'page-link';
    PageLink.innerText := IntToStr(TotalPages);
    PageLink.setAttribute('href', 'javascript:void(0)');
    PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
    var
      PageNum: Integer;
    begin
      PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
      PageNumber := PageNum;
      GetCalls(generateSearchOptions());
    end);
  end;
  PageItem.appendChild(PageLink);
  PaginationElement.appendChild(PageItem);

  // Next Button
  PageItem := TJSHTMLElement(document.createElement('li'));
  PageItem.className := 'page-item';
  if PageNumber = TotalPages then
    PageItem.classList.add('disabled');
  PageLink := TJSHTMLElement(document.createElement('a'));
  PageLink.className := 'page-link';
  PageLink.innerText := 'Next';
  PageLink.setAttribute('href', 'javascript:void(0)');
  PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
  begin
    if PageNumber < TotalPages then
    begin
      Inc(PageNumber);
      GetCalls(GenerateSearchOptions());
    end;
  end);
  PageItem.appendChild(PageLink);
  PaginationElement.appendChild(PageItem);

end;


procedure TFViewOrders.GetCalls(searchOptions: string);
// Retrieves list of calls from the server that meet the criteria searchOptions
// searchOptions: information about how to generate the SQL statement based on
// user input.
var
  xdcResponse: TXDataClientResponse;
  orderList : TJSObject;
  i: integer;
  data: TJSArray;
  order: TJSObject;
  callListLength: integer;
  tempString: string;

begin
  if PageNumber > 0 then
  begin
    asm
      startSpinner();
    end;
    console.log(PageNumber);
    xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.GetOrders',
        [searchOptions]));
    orderList :=  TJSObject(xdcResponse.Result);
    data := TJSArray(orderList['data']);
    callListLength := integer(orderList['count']);
    ClearTable();
    XDataWebDataSet1.Close;
    XDataWebDataSet1.SetJsonData(orderList['data']);
    XDataWebDataSet1.Open;
    console.log(orderList['data']);
    asm
      setTimeout(endSpinner, 2000);
    end;
    for i := 0 to data.Length - 1 do
    begin
      order := TJSObject(data[i]);
      AddRowToTable(tempString);
      XDataWebDataSet1.Next;
    end;
    TotalPages := (callListLength + PageSize - 1) div PageSize;
    if (PageNumber * PageSize) < callListLength then
    begin
      lblEntries.Caption := 'Showing entries ' + IntToStr((PageNumber - 1) * PageSize + 1) +
                            ' - ' + IntToStr(PageNumber * PageSize) +
                             ' of ' + IntToStr(callListLength);
    end
    else
    begin
       lblEntries.Caption := 'Showing entries ' + IntToStr((PageNumber - 1) * PageSize + 1) +
                            ' - ' + IntToStr(callListLength) +
                             ' of ' + IntToStr(callListLength);
    end;
    GeneratePagination(TotalPages);
  end;
end;

procedure TFViewOrders.btnApplyClick(Sender: TObject);
// Button that effectively functions as a GetCalls() button
var
searchOptions: string;
begin
  PageNumber := 1;
  PageSize := StrToInt(wcbPageSize.Text);
  OrderBy := wcbSortBy.Text;
  searchOptions :=  '&pagenumber=' + IntToStr(PageNumber) +
                    '&pagesize=' + IntToStr(PageSize) +
                    '&orderby=' + OrderBy;
  GetCalls(searchOptions);
end;

procedure TFViewOrders.Search(searchOptions: string);
// Search method that searches the database for a specific phone number
var
  xdcResponse: TXDataClientResponse;
  callList : TJSObject;
  i: integer;
  data: TJSArray;
  call: TJSObject;
  callListLength: integer;

begin
  if PageNumber > 0 then
  begin
    xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.Search',
        [searchOptions]));
    callList :=  TJSObject(xdcResponse.Result);
    data := TJSArray(callList['data']);
    callListLength := integer(callList['count']);
    ClearTable();
    for i := 0 to data.Length - 1 do
    begin
      call := TJSObject(data[i]);
      {AddRowToTable(string(call['toNumber']), string(call['fromNumber']), string(call['dateCreated']),
      string(call['duration']), string(call['transcription']), string(call['mediaUrl']));}
    end;
    TotalPages := (callListLength + PageSize - 1) div PageSize;
    lblEntries.Caption := 'Showing entries for phone number: ' + searchOptions;
  end;
end;

procedure TFViewOrders.btnSearchClick(Sender: TObject);
// calls Search method
begin
  Search(edtSearch.Text);
end;

procedure TFViewOrders.ClearTable();
// clears the table
var
  tbody: TJSHTMLElement;
begin
  tbody := TJSHTMLElement(document.getElementById('tblPhoneGrid').getElementsByTagName('tbody')[0]);
  tbody.innerHTML := '';
end;

function TFViewOrders.GenerateSearchOptions(): string;
// Generates searchOptions for get calls.
var
searchOptions: string;
begin
  //PageNumber := 1;
  PageSize := StrToInt(wcbPageSize.Text);
  OrderBy := wcbSortBy.Text;
  searchOptions :=  '&pagenumber=' + IntToStr(PageNumber) +
                    '&pagesize=' + IntToStr(PageSize) +
                    '&orderby=' + OrderBy;
  Result := searchOptions;
end;

end.

