// Auth Database to verify logins

unit Auth.Database;

interface

uses
  System.SysUtils, System.Classes, IniFiles, Vcl.Forms, MemDS,
  Data.DB, DBAccess, Uni, UniProvider, PostgreSQLUniProvider, MySQLUniProvider;

type
  TAuthDatabase = class(TDataModule)
    uq: TUniQuery;
    uqMisc: TUniQuery;
    ucKG: TUniConnection;
    MySQLUniProvider1: TMySQLUniProvider;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure SetLoginAuditEntry( userStr: string );
  end;

var
  AuthDatabase: TAuthDatabase;

implementation

uses
  System.JSON,
  Common.Config,
  Common.Logging,
  uLibrary;

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

procedure TAuthDatabase.DataModuleCreate(Sender: TObject);
var
  IniFile: TIniFile;
  iniStr: string;
begin
  IniFile := TIniFile.Create( ChangeFileExt(Application.ExeName, '.ini') );
  try
    iniStr := IniFile.ReadString( 'Database', 'Server', '' );
    if iniStr.IsEmpty then
      Logger.Log( 1, 'iniFile: ' + ChangeFileExt(Application.ExeName, '.ini') +
                  ' Database->Server: Entry not found' )
    else
    begin
      Logger.Log( 1, 'iniFile: ' + ChangeFileExt(Application.ExeName, '.ini') +
                  ' Database->Server: ' + iniStr );
      ucKG.Server := iniStr;
    end;

    iniStr := IniFile.ReadString( 'Twilio', 'AccountSID', '' );
    if iniStr.IsEmpty then
      Logger.Log( 1, 'iniFile: ' + ChangeFileExt(Application.ExeName, '.ini') +
                  ' Twilio->AccountSID: Entry not found' )
    else
    begin
      Logger.Log( 1, 'iniFile: ' + ChangeFileExt(Application.ExeName, '.ini') +
                  ' Twilio->AccountSID: ' + iniStr );
      //accountSID := iniStr;
    end;

    iniStr := IniFile.ReadString( 'Twilio', 'AuthHeader', '' );
    if iniStr.IsEmpty then
      Logger.Log( 1, 'iniFile: ' + ChangeFileExt(Application.ExeName, '.ini') +
                  ' Twilio->AuthHeader: Entry not found' )
    else
    begin
      Logger.Log( 1, 'iniFile: ' + ChangeFileExt(Application.ExeName, '.ini') +
                  ' Twilio->AuthHeader: ' + iniStr );
      //authHeader := iniStr;
    end;
    Logger.Log(1, '');
  finally
    IniFile.Free;
  end;
end;

procedure TAuthDatabase.DataModuleDestroy(Sender: TObject);
begin
  ucKG.Connected := false;
end;

procedure TAuthDatabase.SetLoginAuditEntry( userStr: string );
var
  auditMasterId: string;
  userInfo: TStringList;
  entry: string;
  username: string;
  fullname: string;
  agency: string;
  userid: string;
  personnelid: string;
  admin: boolean;
  i: Integer;
begin
  Logger.Log( 3, 'TAuthDatabase.SetLoginAuditEntry - start' );

  userInfo := TStringList.Create;
  try
    userInfo.Delimiter := '&';
    userInfo.StrictDelimiter := True;
    userInfo.DelimitedText := userStr;
    username := userInfo.Values['username'];
    fullname := userInfo.Values['fullname'];
    userid := userInfo.Values['userId'];
    personnelid := userInfo.Values['personnelid'];
  finally
    userInfo.Free;
  end;
end;


end.
